#include <windows.h>
#include <winnt.h>

/* CheckReg - Internal function to check registry value */
static int CheckReg(int Server)
{
    int             Result = 0;         /* assume failure       */
    char            szOSType[16] = {0};
    DWORD           dwSize = sizeof( szOSType );
    OSVERSIONINFO   os = { sizeof( os ) };
    HKEY            hKey = NULL;
    LONG            rv;
    
    if( !GetVersionEx( &os ) )  /* if GetVersionEx() fails      */
        Result  = -1;           /* indicate API failure         */
    // If the platform id is not Win32 on Windows NT, this is 
    // neither an NT workstation or server
    else if( os.dwPlatformId != VER_PLATFORM_WIN32_NT )
        ;                       /* FALSE is the default result  */
    // open the ProductOptions subkey
    else if( ERROR_SUCCESS != ( rv = RegOpenKeyEx( HKEY_LOCAL_MACHINE
            , "SYSTEM\\CurrentControlSet\\Control\\ProductOptions"
            , 0, KEY_QUERY_VALUE, &hKey ) ) )
    {
        Result  = -1;           /* indicate API failure         */
        SetLastError( rv );
    }
    else
    {
        /* get the ProductType value                            */
        rv = RegQueryValueEx( hKey, "ProductType"
            , NULL, NULL, (LPBYTE) szOSType, &dwSize );

        /* we're done with the registry                         */
        RegCloseKey( hKey );

        /* if the query of the ProductType value failed,        *
         * set the last saved error                             */
        if( ERROR_SUCCESS != rv )
        {
            Result  = -1;       /* indicate API failure         */
            SetLastError( rv );
        }

        if(!lstrcmpi("ServerNT", szOSType))
            Result  = Server;
        else if(!lstrcmpi("LanmanNT", szOSType))
            Result  = Server;
        else if(!lstrcmpi("WinNT", szOSType))
            Result  = !Server;
    }

    /* if the answer is "no" but there was no API failure */
    if(Result == 0)
        SetLastError( ERROR_SUCCESS );

    return Result;
}

/* return TRUE when OS is Windows NT Workstation */
BOOL IsNTWorkstation(void)
{
    return CheckReg(FALSE) > 0;    
}
/* return TRUE when OS is Windows NT Server */
BOOL IsNTServer(void)
{
    return CheckReg(TRUE) > 0;    
}


/* change the following line to "#if 1" to build a small test program.
 *
 * Visual C++:  cl -DSTRICT /W3 whatami.c advapi32.lib
 * Borland C++: bcc32 -w -DSTRICT whatami.c
 */
#if 0

#include <stdio.h>

void main(void)
{
    if( IsNTWorkstation() )
        printf( 
         "You are running WhatAmI.exe on Windows NT Workstation." );
    else
    if( IsNTServer() )
        printf( 
         "You are running WhatAmI.exe on Windows NT Server." );
    else if(GetLastError() != ERROR_SUCCESS)
        printf("An error was encountered.\n");
    else
        printf( 
         "You are not running WhatAmI.exe on an NT platform." );
}
#endif  /* end of small test program */
